#!/usr/bin/perl

#
# vdradmin.pl by Thomas Koch <no-return@gmx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Or, point your browser to http://www.gnu.org/copyleft/gpl.html
#
# 08.10.2001
#

use lib qw(lib/);
use CGI;
use IO::Socket;
use HTML::Template;
use Time::Local;
use POSIX;

use strict;

my($VDRHOST)          = "video";
my($FILEPREFIX)       = "/tmp";

my($VDRPORT)          = 2001;
my($LANG)             = "de";
my($LOCALEPGFILE)     = "$FILEPREFIX/epg.data";
my($LOCALCHANFILE)    = "$FILEPREFIX/chanlist";
my($LOCALLASTGETFILE) = "$FILEPREFIX/lastget";
my($LOCALEPGTIMEOUT)  = 360;
my($TEMPLATEDIR)      = "template";

#############################################################################
#
sub ReadLine {
    my($Socket) = @_;
    my($ret, $buf, $RecvStr);
    while(1) {
	$ret = read($Socket, $buf, 1);
        if($buf eq "\n") { return("$RecvStr\n"); }
	if($buf ne "\r") { $RecvStr .= $buf; }
	if($ret == 0 || $ret == undef) { return($RecvStr); }
    }
}


#############################################################################
#
sub SendLine {
    my($Socket, $cmd) = @_;
    my($result) = send($Socket, $cmd, 0);
    if($result != length($cmd)) {
	die("Error sending command to host $VDRHOST\n");
    }
}

#############################################################################
#
sub SVDRPSend {
    my($cmd) = @_;
    my($Socket) = IO::Socket::INET->new(PeerAddr => $VDRHOST, PeerPort => $VDRPORT, Proto => 'tcp');
    my(@output);

    ReadLine($Socket);
    SendLine($Socket, $cmd);
    
    my($first) = 1; my($last);
    while(<$Socket>) {
	if($first) { $last = substr($_, 0, 3); }
	s/\r//g;
	if(substr($_, 3, 1) eq "-") {
	    s/^$last-//;
	    push(@output, $_);
	} else {
	    s/^$last //;
	    push(@output, $_);
	    last;
	}
    }
    SendLine("quit\r\n");
    close($Socket);

    return(@output);
}


#############################################################################
#
sub Get {    
    open(FILE, ">$LOCALEPGFILE") || HTMLError("can't open $LOCALEPGFILE for write!\n");
    for(SVDRPSend("lste\r\n")) { print(FILE $_); }
    close(FILE);

    open(FILE, ">$LOCALCHANFILE") || HTMLError("can't open $LOCALCHANFILE for write!\n");
    for(SVDRPSend("lstc\r\n")) { print(FILE $_); }
    close(FILE);
}


#############################################################################
# Getting EPG data from SVDRP takes much time and VDR does not allow
# several SVDRP connections at one time.
sub UptoDate {
    open(TIMEOUT, $LOCALLASTGETFILE);
    my($lastget) = <TIMEOUT>;
    close(TIMEOUT);
    if((time - $lastget) >= $LOCALEPGTIMEOUT) {
	Get();
	open(TIMEOUT, ">$LOCALLASTGETFILE") || HTMLError("can't open $LOCALLASTGETFILE for write!\n");
	print(TIMEOUT time);
	close(TIMEOUT);
    }
}


#############################################################################
#
sub GetChannelDesc {
    my(%hash);
    open(FILE, $LOCALCHANFILE);
    while(<FILE>) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	if(length($desc) > 12) {
	    $desc = substr($desc, 0, 12);
	    $desc .= "...",
	}
	$hash{$id} = $desc;
    }
    close(FILE);
    return(%hash);
}


#############################################################################
#
sub EURL {
    my($text) = @_;
    $text =~ s/([^0-9a-zA-Z])/sprintf("%%%2.2x", ord($1))/ge;
    return($text);
}


#############################################################################
#
sub GetChannelID {
    my(%hash);
    for(SVDRPSend("lstc\r\n")) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	$hash{$nnumber} = $id;
    }
    return(%hash);
}


#############################################################################
#
sub HTMLError {
    my($error) = @_;   
    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_error.html");
    $template->param(error => $error);
    print("Content-type: text/html\n\n", $template->output);
    exit(1);
}


my($q) = CGI->new;


#############################################################################
#
if($q->param('aktion') eq "prog_detail") {
    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_prog_detail.html");
    $template->param(
	titel => $q->param('titel') ? $q->param('titel') : undef, 
	text  => $q->param('text') ? $q->param('text') : undef,
    );
    print("Content-type: text/html\n\n", $template->output); 
}


#############################################################################
#
if($q->param('aktion') eq "prog_list") {

    UptoDate();
    
    if(!$q->param('number')) {
	open(FILE, $LOCALCHANFILE) || HTMLError("can't open $LOCALCHANFILE for read!");
	my($desc, $nnumber);
	while(<FILE>) {
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    ($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	    last;
	}
	close(FILE);
	print($q->redirect($q->url . "?aktion=prog_list&number=$nnumber"));
	exit(0);
    } 
    
    my($number) = $q->param('number');
    my(@temp, @show, $progname, $channumber);
    
    open(VDRFILE, $LOCALEPGFILE) || HTMLError("can't open $LOCALEPGFILE for read!");
    while(<VDRFILE>) {
	chomp;
	if(/^C ([^ ]+) *(.*)/) {
	    if($1 != $number) { next; } else { $progname = $2; $channumber = $1}
	    while(<VDRFILE>) {
		if(/^E (.*) (.*) (.*)/) {
		    my($time); my($duration);
		    my($time, $duration) = ($2, $3);
		    my($title, $subtitle, $description);
		    while(<VDRFILE>) {
                        if (/^T (.*)/) { $title = $1 }
			if (/^S (.*)/) { $subtitle = $1 }
			if (/^D (.*)/) { $description = $1 }
			if (/^e/) {
			    push(@temp, {
				ssse        => $time,
				emit        => strftime("%H:%M", localtime($time)),
				duration    => strftime("%H:%M", localtime($time + $duration)),
				title       => $title,
				subtitle    => $subtitle,
				recurl => $q->url . "?aktion=timer_new_form&active=1&channumber=" . $channumber . "&dor=" . strftime("%d", localtime($time)) . "&starth=" . strftime("%H", localtime($time)) . "&startm=" . strftime("%M", localtime($time)) . "&endh=" . strftime("%H", localtime($time + $duration)) . "&endm=" . strftime("%M", localtime($time + $duration)) . "&prio=99&lft=99&name=" . EURL($title),
				infurl      => $description ? $q->url . "?aktion=prog_detail&titel=" . EURL($title) . "&text=" . EURL($description) : undef,
				toggle      => undef
			    });
			    last;
			}
		    }
		} elsif(/^c/) {
		    last;    
		}
	    }
	}
    }
    # FIXME: Sometimes VDR has all events twice
    my($last); my($toggle) = 0; my(@show);
    for(sort({ $a->{ssse} <=> $b->{ssse} } @temp)) {
	next if($last eq $_->{title});
	if($toggle) { $toggle = 0 } else { $toggle = 1 }
	$_->{toggle} = $toggle;
	push(@show, $_);
	$last = $_->{title};
    }
    close(VDRFILE);
    
   
    #
    my(@chans);
    open(FILE, $LOCALCHANFILE) || HTMLError("can't open $LOCALCHANFILE for read!");
    while(<FILE>) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	if(length($desc) > 12) {
	    $desc = substr($desc, 0, 12);
	    $desc .= "...",
	}
	push(@chans, {
	    name => $desc,
	    url  => $q->url . "?aktion=prog_list&number=$nnumber",
	    cur  => ($number == $nnumber) ? 1 : 0,
	});
    }
    close(FILE);

    # 
    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_prog_list.html");
    $template->param(
	url => $q->url,
	loop => \@show,
	chanloop => \@chans,
	progname => $progname
    );
    print("Content-type: text/html\n\n", $template->output);

}


#############################################################################
#
if($q->param('aktion') eq "timer_list") {

    my(@timers);
    my(%chandesc) = GetChannelDesc();
    
    for(SVDRPSend("lstt\n")) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);
	my($active, $prgnum, $dor, $start, $stop, $prio, $lft, $name, $summary) = split(/\:/, $temp, 9);
	push(@timers, {
	    active => $active ? "Yes" : "No",
	    prgnum => $chandesc{$prgnum},
	    dor    => $dor,
	    start  => substr($start, 0, 2) . ":" . substr($start, 2, 2),
	    stop   => substr($stop, 0, 2) . ":" . substr($stop, 2, 2),
	    prio   => $prio,
	    lft    => $lft,
	    name   => $name,
	    sum    => $summary,
	    delurl => $q->url . "?aktion=timer_delete&id=$id",
	    modurl => $q->url . "?aktion=timer_new_form&reedit=$id&chan=" . $prgnum . "&active=" . $active . "&dor=$dor&starth=" . substr($start, 0, 2) . "&startm=" . substr($start, 2, 2) . "&endh=" . substr($stop, 0, 2) . "&endm=" . substr($stop, 2, 2) . "&prio=$prio&lft=$lft&name=" . EURL($name) . "&summary=" . EURL($summary),
	});
    }


    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_timer_list.html");
    $template->param(
	loop => \@timers,
	nturl  => $q->url . "?aktion=timer_new_form",
    );
    print("Content-type: text/html\n\n", $template->output);
}


#############################################################################
#
if($q->param('aktion') eq "timer_new_form") {

    #
    my(@chans);
    open(FILE, $LOCALCHANFILE) || HTMLError("can't open $LOCALCHANFILE for read!"); 
    while(<FILE>) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	next if(!$vpid); # VDR can't record Radio program's
	next if(!$nnumber); # Whats that? Go away!
	if(length($desc) > 12) {
	    $desc = substr($desc, 0, 12);
	    $desc .= "...",
	}
	push(@chans, {
	    name => $desc,
	    id   => $id,
	    cur  => ($nnumber == $q->param('channumber') || $id == $q->param('chan')) ? 1 : 0,
	});
    }
    close(FILE);

    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_timer_new.html");
    $template->param(
	titel  => "Neuer Timer",
	url    => $q->url,
	chans  => \@chans,
	active => $q->param('active') ? $q->param('active') : 0,
	dor    => $q->param('dor') ? $q->param('dor') : undef,
	starth => $q->param('starth') ? $q->param('starth') : undef,
	startm => $q->param('startm') ? $q->param('startm') : undef,
	endh   => $q->param('endh') ? $q->param('endh') : undef,
	endm   => $q->param('endm') ? $q->param('endm') : undef,
	prio   => $q->param('prio') ? $q->param('prio') : 99,
	lft    => $q->param('lft') ? $q->param('lft') : 99,
	title  => $q->param('name') ? $q->param('name') : undef,
	summary=> $q->param('summary') ? $q->param('summary') : undef,
	reedit => $q->param('reedit') ? $q->param('reedit') : 0
    );
    print("Content-type: text/html\n\n", $template->output);
}


#############################################################################
#
if($q->param('aktion') eq "timer_add") {
    
    my($start, $end, $prio, $lft, $title);

    #
    my($dor) = $q->param('dor');
    
    #
    if($q->param('starth') =~ /\d+/ && $q->param('starth') < 24 && $q->param('starth') >= 0) {
	if($q->param('startm') =~ /\d+/ && $q->param('startm') < 60 && $q->param('starth') >= 0) {
	    $start = $q->param('starth') . $q->param('startm');
	} else {
	    HTMLError("Error: Bitte Startzeit richtig eingeben!");
	}
    } else { 
	HTMLError("Error: Bitte Startzeit richtig eingeben!");
    }
    
    #
    if($q->param('endh') =~ /\d+/ && $q->param('endh') < 24 && $q->param('endh') >= 0) {
	if($q->param('endm') =~ /\d+/ && $q->param('endm') < 60 && $q->param('endm') >= 0) {
	    $end = $q->param('endh') . $q->param('endm');
	} else {
	    HTMLError("Error: Bitte Endzeit richtig eingeben!");
	}
    } else { 
	HTMLError("Error: Bitte Endzeit richtig eingeben!");
    }

    
    #
    if($q->param('prio') =~ /\d\d/) {
	$prio = $q->param('prio');
    } else {
	HTMLError("Error: Bitte Prioritaet richtig eingeben!");
    }

    #
    if($q->param('lft') =~ /\d\d/) {
	$lft = $q->param('lft');
    } else {
	HTMLError("Error: Bitte Lifetime richtig eingeben!");
    }
    
    #
    if($q->param('title')) {
	$title = $q->param('title');
	$title =~ s/\://g;
    } else {
	HTMLError("Error: Bitte Titel eingeben!");
    }
    
    if(!$q->param('reedit')) {
	SVDRPSend("newt " . $q->param('active') . ":" . $q->param('chans') . ":" . $dor . ":" . $start . ":" . $end  . ":" . $prio . ":" . $lft . ":" . $title . ":" . $q->param('summary') . "\n");
    } else {
	SVDRPSend("modt " . $q->param('reedit') . " " . $q->param('active') . ":" . $q->param('chans') . ":" . $dor . ":" . $start . ":" . $end  . ":" . $prio . ":" . $lft . ":" . $title . ":" . $q->param('summary') . "\n");
    }
    
    print($q->redirect($q->url . "?aktion=timer_list"));
}


#############################################################################
#
if($q->param('aktion') eq "timer_delete") {
    my($id) = $q->param('id');

    my($result) = SVDRPSend("modt $id off\ndelt $id\n");
#    if($result !~ /Timer "$id" deleted/) {
#	HTMLError("Timer konnte nicht geloescht werden. $result");
#	exit(1);
#    }
    print($q->redirect($q->url . "?aktion=timer_list"));
}


#############################################################################
#
if($q->param('aktion') eq "prog_summary") {

    UptoDate();
      
    my(%chans) = GetChannelID();
    my(@temp, @show, $progname);
    my($rows) = 0; my($maxrows) = 2;
    my($eventtime, %temp, $temp);
    
    if($q->param('time')) {
	my($hour, $minute) = split(/\:/, $q->param('time'), 2);
	if($hour < strftime("%H", localtime(time))) {
	    $eventtime = timelocal(undef, $minute, $hour, strftime("%d", localtime(time + 86400)), (strftime("%m", localtime(time + 86400)) - 1), strftime("%Y", localtime(time))) + 1;
	} else {
	    $eventtime = timelocal(undef, $minute, $hour, strftime("%d", localtime(time)), (strftime("%m", localtime(time)) - 1), strftime("%Y", localtime(time))) + 1;
	} 
    } else {
	$eventtime = time;
    }
    
    #
    my(@chans);
    open(FILE, $LOCALCHANFILE) || HTMLError("can't open $LOCALCHANFILE for read!");
    while(<FILE>) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	(my($radio) = 1) if(!$vpid);
	if(length($desc) > 12) {
	    $desc = substr($desc, 0, 12);
	    $desc .= "...",
	}
	push(@chans, {
	    name => $desc,
	    radio=> $radio,
	    url  => $q->url . "?aktion=prog_list&number=$nnumber",
	    number => $nnumber
	});
    }
    close(FILE);
    
    
    #
    my($i) = 1; my($ee);
    open(VDRFILE, $LOCALEPGFILE) || HTMLError("can't open $LOCALEPGFILE for read!");
    while(<VDRFILE>) {
	chomp;
	if(/^C ([^ ]+) *(.*)/) {
	    my($found) = 0; my($proglink); my($chan); 
	    if($ee) { $i++; $ee = 0; }

	    for $chan (@chans) {
		if($chan->{number} == $1) { 
		    if($chan->{radio}) { $found = 1 }
		    $proglink = $chan->{url};
		}
	    }
	    if($found) { next; }
	    my($progname) = $2; my($channumber) = $1; my($first) = 1;
            while(<VDRFILE>) {
		if(/^E (.*) (.*) (.*)/) {
		    my($time); my($duration);
		    my($time, $duration) = ($2, $3);
		    if($time < $eventtime && $eventtime > ($duration + $time)) { next; }
		    my($title); my($subtitle); my($description);
		    while(<VDRFILE>) {
                        if (/^T (.*)/) { $title = $1 }
			if (/^S (.*)/) { $subtitle = $1 }
			if (/^D (.*)/) { $description = $1 }
			if (/^e/) {
			    if($temp[$i]->{sse} > $time || $first) {
				$temp[$i] = {
				    sse        => $time,
				    prog       => $progname,
				    date       => strftime("%d.%m.%Y", localtime($time)),
				    emit       => strftime("%H:%M", localtime($time)),
				    duration   => strftime("%H:%M", localtime($time + $duration)),
				    title      => $title ? $title : undef,
				    subtitle   => (length($subtitle) > 30) ? substr($subtitle, 0, 30) . "..." : $subtitle,
				    summary    => $description ? substr($description, 0, 120) . "... " : undef,
				    infurl     => $description ? $q->url . "?aktion=prog_detail&titel=" . EURL($progname . ' | ' . strftime("%H:%M", localtime($time)) . '-' . strftime("%H:%M", localtime($time + $duration)) . ' | ' . $title) . "&text=" . EURL($description) : undef,
				    recurl     => $q->url . "?aktion=timer_new_form&active=1&channumber=" . $channumber . "&dor=" . strftime("%d", localtime($time)) . "&starth=" . strftime("%H", localtime($time)) . "&startm=" . strftime("%M", localtime($time)) . "&endh=" . strftime("%H", localtime($time + $duration)) . "&endm=" . strftime("%M", localtime($time + $duration)) . "&prio=99&lft=99&name=" . EURL($title),
				    proglink   => $proglink,
				    row0       => undef,
				    row1       => undef,
				
				};
				$first = 0;
				$ee = 1;
			    }
			    last;
			}
	    	    }		    
		} elsif(/^c/) {
		    last;
		}
	    }
	}
    }
    close(VDRFILE);
    # FIXME: Sometimes VDR has all events twice
    my($last); my($toggle) = 0; 
    for(@temp) {
	next if($last eq $_->{title});
	($rows == 0) ? ($_->{row0} = 1) : ($_->{row0} = 0);
	($rows == $maxrows) ? ($_->{row1} = 1) : ($_->{row1} = 0);
	push(@show, $_);
	$last = $_->{title};
	if($rows < $maxrows) {
	    $rows++;
	} else {
	    $rows = 0;
	}
    }
    if($rows) {
	for(; $rows <= $maxrows; $rows++) {
	    if($rows == $maxrows) {	
		push(@show, { row1 => 1 });
	    } else {
		push(@show, { row0 => undef });
	    }
	}
    }
   

    # 
    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_prog_summary.html");
    $template->param(
	rows   => \@show,
	now    => strftime("%H:%M", localtime($eventtime)),
	nowurl => $q->url . "?aktion=prog_summary",
	url    => $q->url
    );
    print("Content-type: text/html\n\n", $template->output);

}


#############################################################################
#
if($q->param('aktion') eq "rec_list") {
    
    my(@temp);
    
    for(SVDRPSend("lstr\r\n")) {
	chomp;
	my($new);
	my($id, $date, $time, $name) = split(/ +/, $_, 4);
	
	if(substr($time, 5, 1) eq "*") {
	    $new = 1;
	    $time =~ s/\*//;
	}

	push(@temp, {
	    date   => $date,
	    time   => $time,
	    name   => $name,
	    new    => $new,
	    delurl => $q->url . "?aktion=rec_delete&id=$id",
	});
    }
    my(@recordings) = sort({ $a->{date} <=> $b->{date} } @temp);
    
    my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/${LANG}_rec_list.html");
    $template->param(recloop => \@recordings);
    print("Content-type: text/html\n\n", $template->output);

}


#############################################################################
#
if($q->param('aktion') eq "rec_delete") {
    my($id) = $q->param('id');
    SVDRPSend("delr $id\r\n");
    print($q->redirect($q->url . "?aktion=rec_list"));
}

# EOF
